-------------------------------------------------------------------------------
-- Addon namespace
local nanoTalk = LibStub("AceAddon-3.0"):NewAddon("nanoTalk", "AceEvent-3.0", "AceTimer-3.0")
local LDB = LibStub:GetLibrary("LibDataBroker-1.1")
local LQT = LibStub:GetLibrary("LibQTip-1.0")

nanoTalk.dataobj = LDB:NewDataObject("nanoTalk", {
			type	= "data source",
			icon	= "Interface\\GossipFrame\\GossipGossipIcon",
			label	= "nanoTalk",
			text	= "0t",
		})


_G["nanoTalk"] = nanoTalk


-------------------------------------------------------------------------------
-- Localized globals
local ipairs, pairs = ipairs, pairs
local tremove, tinsert, tsort = table.remove, table.insert, table.sort


-------------------------------------------------------------------------------
-- Local variables
local version = GetAddOnMetadata("nanoTalk", "Version")
local db
local defaults = {
	profile = {
		ChatNotify = true,
		NotifyAFK = true,
		TalkTime = 120,
		TrackNum = 10,
		TrackTime = 1800,
		Version = version
	},
	factionrealm = {
		History = {}
	},
}
local timers = {}
local updatedLines = {}
local timestamps = {}
local people = {}
local AFKtime = 0
local tooltip = LQT:Acquire("nanoTalk", 3, "LEFT", "CENTER", "CENTER")

nanoTalk.Timers = timers


-------------------------------------------------------------------------------
-- Local functions
local function GetCount(timer)
	if (timer == nil) then
		return #timestamps
	end

	local count = 0

	for _, tm in ipairs(timestamps) do
		if ((GetTime() - tm) <= timer) then
			count = count + 1
		end
	end
	return count
end


local function TimeStr(seconds)
	local tm = seconds
	local timeval = tostring(math.floor(tm % 60)).."s"

	if (tm >= 60) then
		tm = tm / 60
		timeval = tostring(math.floor(tm % 60)).."m"..timeval

		if (tm >= 60) then
			tm = tm / 60
			timeval = tostring(math.floor(tm % 24)).."h"..timeval

			if (tm >= 24) then
				tm = tm / 24
				timeval = tostring(math.floor(tm)).."d"..timeval
			end
		end
	end
	return timeval
end


local function ShowTooltip()
	tooltip:Clear()
	tooltip:AddHeader("nanoTalk")
	tooltip:AddLine(" ")

	local count = GetCount(db.TrackTime)
	local linenum

	if (count > 0) then
		tooltip:AddHeader("|cffff9933Name|r", "|cffff9933Messages|r", "|cffff9933Last|r")

		for _, tm in ipairs(timestamps) do
			local curtm = GetTime() - tm

			if (curtm <= db.TrackTime) then
				linenum = tooltip:AddLine(people[tm].name, people[tm].msgs, TimeStr(curtm))
				updatedLines[linenum] = tm
			end
		end
		tooltip:AddLine(" ")
		linenum = tooltip:AddLine("Left Click:")
		tooltip:SetCell(linenum, 2, "Clear list.")
	else
		tooltip:AddLine("No messages.")
		tooltip:AddLine(" ")
	end
	linenum = tooltip:AddLine("Right Click:")
	tooltip:SetCell(linenum, 2, "Configure.")
	tooltip:Show()
end


local function UpdateTooltip()
	if (tooltip:IsShown()) then
		updatedLines = {}
		ShowTooltip()
	end
end


function nanoTalk:Clear()
	timestamps = {}
	people = {}
	UpdateTooltip()
end


-------------------------------------------------------------------------------
-- Initialization and event handlers
function nanoTalk:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("nanoTalkConfig", defaults)
	self.db.RegisterCallback(self, "OnProfileChanged", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileCopied", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileReset", "OnProfileChanged")
	db = self.db.profile

--	self:CreateFrames()
	self:SetupOptions()
end


function nanoTalk.dataobj:OnClick(button)
	if (button == "LeftButton") then
		nanoTalk:Clear()
	else
		InterfaceOptionsFrame_OpenToCategory(nanoTalk.optionsFrames.nanoTalk)
	end
end


function nanoTalk.dataobj.OnLeave()
	tooltip:Hide()
	updatedLines = {}
end


function nanoTalk.dataobj.OnEnter(self)
	tooltip:SmartAnchorTo(self)
	ShowTooltip()
end


function nanoTalk:OnEnable()
	self:RegisterEvent("CHAT_MSG_WHISPER")
	timers.UpdateInfo = self:ScheduleRepeatingTimer("UpdateInfo", 0.5)
end


function nanoTalk:OnDisable()
	self:UnregisterEvent("CHAT_MSG_WHISPER")

	for k, v in pairs(timers) do
		self:CancelTimer(v, true)
		timers[k] = nil
	end
end


function nanoTalk:CHAT_MSG_WHISPER(event, msg, player)
	local stamp = GetTime()
	local last = nil

	for tm, plr in pairs(people) do
		if (plr.name == player) then
			last = tm
			break
		end
	end

	local msgcnt = 0

	if (last ~= nil) then
		msgcnt = people[last].msgs
		people[last] = nil

		for idx, tm in ipairs(timestamps) do
			if (tm == last) then
				tremove(timestamps, idx)
				break
			end
		end
	end

	people[stamp] = {
		name = player,
		msgs = msgcnt + 1
	}
	tinsert(timestamps, stamp)

	if (#timestamps > 1) then tsort(timestamps) end

	if ((db.NotifyAFK == true) and UnitIsAFK("player")) then
		SendChatMessage("I have been AFK for "..TimeStr(GetTime() - AFKtime)..".", "WHISPER", nil, player)
	elseif ((last == nil) or ((stamp - last) > db.TalkTime)) then
		local count = GetCount(db.TalkTime) - 1

		if (count < 1) then UpdateTooltip() return end

		if (db.ChatNotify == true) then
			local phrase = "Someone else is"

			if (count > 1) then
				phrase = "I have "..count.." other people"
			end

			SendChatMessage("Hello, "..player.."! "..phrase.." talking to me, so I might not answer right away.", "WHISPER", nil, player)
			ChatFrame1:AddMessage("|cffff9933nanoTalk:|r Informed "..player.." of conversation.")
		end
	end
	UpdateTooltip()
end


function nanoTalk:UpdateInfo()
	if (#timestamps > db.TrackNum) then
		local ppos = tremove(timestamps)
		people[ppos] = nil
	end

	if (AFKtime == 0) then
		if (UnitIsAFK("player")) then
			AFKtime = GetTime()
		end
	elseif (not UnitIsAFK("player")) then
		ChatFrame1:AddMessage("|cffff9933nanoTalk:|r You were AFK for "..TimeStr(GetTime() - AFKtime)..".")
		AFKtime = 0
	end

	local total = GetCount(db.TrackTime)
	local recent = GetCount(db.TalkTime)

	if (recent > 0) then
		self.dataobj.text = ""..total.."t "..recent.."r"
		self.dataobj.icon = "Interface\\GossipFrame\\PetitionGossipIcon"
	else
		self.dataobj.text = ""..total.."t "
		self.dataobj.icon = "Interface\\GossipFrame\\GossipGossipIcon"
	end

	if (tooltip:IsShown() and #updatedLines) then
		for linenum,tm in pairs(updatedLines) do
			tooltip:SetCell(linenum, 3, TimeStr(GetTime() - tm))
		end
	end
end


function nanoTalk:OnProfileChanged(event, database, newProfileKey)
	db = database.profile
end
